(* $Id: godi_win32.ml 805 2008-06-11 00:50:03Z gerd $  
 * ----------------------------------------------------------------------
 * Copyright 2008 by Gerd Stolpmann.
 *
 * This file is part of godi-tools.
 *
 * godi-tools is free software; you can redistribute it and/or modify it
 * under the conditions in the file LICENSE.
 *
 * godi-tools is distributed WITHOUT ANY WARRANTY, as detailed in the
 * file LICENSE.
*)

type reg_handle

type reg_value =
  | RV_BINARY of string
  | RV_DWORD of int32
  | RV_EXPAND_LZ of string
  | RV_LINK of string
  | RV_MULTI_SZ of string
  | RV_NONE
  | RV_QWORD of int64
  | RV_SZ of string

type reg_type =
  | RT_BINARY
  | RT_DWORD
  | RT_EXPAND_LZ
  | RT_LINK
  | RT_MULTI_SZ
  | RT_NONE
  | RT_QWORD
  | RT_SZ

exception Registry_error of string

let _ = 
  Callback.register_exception
    "Godi_win32.Registry_error" (Registry_error "")

external get_root_key : int -> reg_handle = "godiwin32_get_root_key"

let hkey_classes_root = get_root_key 0
let hkey_current_user = get_root_key 1
let hkey_local_machine = get_root_key 2
let hkey_users = get_root_key 3

external reg_open_key : reg_handle -> string -> reg_handle
  = "godiwin32_reg_open_key"

(*external reg_open_current_user : unit -> reg_handle
  = "godiwin32_reg_open_current_user"
 *)

external reg_close_key : reg_handle -> unit
  = "godiwin32_reg_close_key"

external reg_query_value_1 : reg_handle -> string -> reg_value
  = "godiwin32_reg_query_value"

let strip_null s =
  let l = String.length s in
  if l > 0 && s.[l-1] = '\000' then
    String.sub s 0 (l-1)
  else
    s

let reg_query_value hkey name =
  match reg_query_value_1 hkey name with
    | RV_SZ data ->
	RV_SZ(strip_null data)
    | RV_EXPAND_LZ data ->
	RV_EXPAND_LZ(strip_null data)
    | other ->
	other

external reg_enum_key_1 : reg_handle -> (string * string) list
  = "godiwin32_reg_enum_key"

external reg_enum_value_1 : reg_handle -> (string * reg_type) list
  = "godiwin32_reg_enum_value"

let reg_enum_key hkey = List.rev (reg_enum_key_1 hkey)
let reg_enum_value hkey = List.rev (reg_enum_value_1 hkey)

external get_logical_drives : unit -> int
  = "godiwin32_get_logical_drives"

external get_identity_information : string -> (int32 * int64)
  = "godiwin32_get_identity_information"

external sleep : float -> unit
  = "godiwin32_sleep"

external interrupt_sleep : unit -> unit
  = "godiwin32_interrupt_sleep"

external generate_ctrl_c : unit -> unit
  = "godiwin32_generate_ctrl_c"

external get_process_id : int -> int
  = "godiwin32_get_process_id"

external peek_pipe : Unix.file_descr -> int
  = "godiwin32_peek_pipe"

external string_of_fd : Unix.file_descr -> string
  = "godiwin32_string_of_fd"
